import serial
import aiohttp
import requests
import time
import os
import asyncio
import websockets

from dotenv import load_dotenv

load_dotenv()


api_url = os.getenv('API_URI')
calculer_consommation =  api_url + '/calculer-volume-consommation'
methode_notification_url = api_url + '/methode-notification'

temps_recuperation_method_notification = os.getenv('TEMPS_RECUPERATION_METHOD_NOTIFICATION') # en secondes
temps_recuperation_distance = os.getenv('TEMPS_RECUPERATION_DISTANCE') # en secondes

arduino_port = '/dev/ttyACM0'
ser = serial.Serial(arduino_port, 9600, timeout=1)

async def process_distance(distance):
    payload = {'distance': distance}
    try:
        async with aiohttp.ClientSession() as session:
            async with session.post(calculer_consommation, json=payload) as response:
                response.raise_for_status()
                data = await response.json()
                print(f"API Response: {data}")
    except aiohttp.ClientError as e:
        print("===", calculer_consommation, "====", api_url)
        print(f"Erreur lors de l'appel API : {e}")

async def process_notification(message):
    print(f"Notification reçue : {message}")

    notification_methode = message
    print(f"Nouvelle méthode de notification reçue: {notification_methode}")
    ser.write(bytes(notification_methode,  'utf-8'))

async def read_serial():
    while True:
        line = ser.readline().decode('utf-8').rstrip()
        print(f"Line : {line}")

        if 'Distance' in line:
            distance = float(line.split(':')[-1].strip().split(' ')[0])
            print(f"Distance : {distance} cm")
            await process_distance(distance)

        # attendre 1 secondes
        await asyncio.sleep(0.1)

async def message_recu(websocket):
    while True:
        # Attendre un message du serveur WebSocket
        message = await websocket.recv()

        # Traitement du message reçu
        await process_notification(message)

async def main():
    async with websockets.connect('ws://localhost:3000') as websocket:
        try:
            print("Connexion au serveur WebSocket établie.")

            # Utilisation de asyncio.create_task pour démarrer les coroutines en parallèle
            task1 = asyncio.create_task(message_recu(websocket))
            task2 = asyncio.create_task(read_serial())

            # Attendre que les deux tâches soient terminées
            await asyncio.gather(task1, task2)

        except KeyboardInterrupt:
            ser.close()
            print("Arrêt du programme.")

if __name__ == "__main__":
    asyncio.get_event_loop().run_until_complete(main())
